/**
 * JHOVE2 - Next-generation architecture for format-aware characterization
 *
 * Copyright (c) 2009 by The Regents of the University of California,
 * Ithaka Harbors, Inc., and The Board of Trustees of the Leland Stanford
 * Junior University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * o Neither the name of the University of California/California Digital
 *   Library, Ithaka Harbors/Portico, or Stanford University, nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.jhove2.module.format.root;

import java.io.EOFException;
import java.io.IOException;
import java.io.*;
import java.util.List;
import java.util.ArrayList;
import java.lang.Runtime;

import org.jhove2.annotation.ReportableProperty;
import org.jhove2.core.JHOVE2;
import org.jhove2.core.JHOVE2Exception;
import org.jhove2.core.format.Format;
import org.jhove2.core.source.RootSource;
import org.jhove2.core.source.Source;
import org.jhove2.module.format.BaseFormatModule;

import org.jhove2.core.Message;
import org.jhove2.core.Message.Context;
import org.jhove2.core.Message.Severity;
import org.jhove2.config.ConfigInfo;

//import hep.io.root.*;
//import hep.io.root.interfaces.*;


/**
 * JHOVE2 Root module.
 * 
 * @author skalinin
 */
public class RootModule
	extends BaseFormatModule

{
	/** Directory module version identifier. */
	public static final String VERSION = "1.9.5";

	/** Directory module release date. */
	public static final String RELEASE = "2010-02-16";

	/** Directory module rights statement. */
	public static final String RIGHTS = "BUW Copyright 2010 by The Regents of the University of California, "
			+ "Ithaka Harbors, Inc., and The Board of Trustees of the Leland "
			+ "Stanford Junior University. "
			+ "Available under the terms of the BSD license.";
	/** List of keys stored in the file. */
	//protected List<String> keys;
	
	/** List of branches names. */
	//protected List<String> branches;

	/** List of leaves names. */
	//protected List<String> leaves;

	/** List of lines from ROOT output */
	protected List<String> root_output;

	/**
	 * Instantiate a new <code>RootModule</code>.
	 * 
	 * @param format
	 *            Root format
	 */
	public RootModule(Format format) {
		super(VERSION, RELEASE, RIGHTS, Scope.Generic, format);

		/*this.keys	= new ArrayList<String>();
		this.branches	= new ArrayList<String>();
		this.leaves	= new ArrayList<String>();*/
		this.root_output= new ArrayList<String>();
	}

	/**
	 * Parse a Root source unit.
	 * 
	 * @param jhove2
	 *            JHOVE2 framework
	 * @param source
	 *            Root source unit
	 * @return 0
	 * @throws EOFException
	 *             If End-of-File is reached reading the source unit
	 * @throws IOException
	 *             If an I/O exception is raised reading the source unit
	 * @throws JHOVE2Exception
	 * @see org.jhove2.module.format.FormatModule#parse(org.jhove2.core.JHOVE2,
	 *      org.jhove2.core.source.Source)
	 */
	@Override
	public long parse(JHOVE2 jhove2, Source source)
	    throws EOFException, IOException, JHOVE2Exception
	{
		if (source instanceof RootSource) {
			List<Source> children = ((RootSource) source)
					.getChildSources();
			for (Source src : children) {
				jhove2.characterize(src);
			}
		}
		/*RootFileReader rfr = new RootFileReader(source.getFile().getName());
		int keys=rfr.nKeys();
		System.out.println("Number of keys: "+ keys);
		try{
		    for(int i=0;i<rfr.nKeys();i++){
			System.out.println("Key "+rfr.getKey(i).getName());
			this.keys.add(rfr.getKey(i).getName());
			TTree tree = (TTree) rfr.get(rfr.getKey(i).getName());
			TObjArray branches = tree.getBranches();
			for(int j=0;j<branches.size();j++){
			    TBranch branch = (TBranch) branches.getElementAt(j);
			    System.out.println("\t Branch "+branch.getName());
			    this.branches.add(branch.getName());
			    TObjArray leaves = branch.getLeaves();
			    for(int k=0;k<leaves.size();k++){
				TLeaf leaf = (TLeaf) leaves.getElementAt(k);
				System.out.println("\t\t Leaf "+leaf.getName());
				this.leaves.add(leaf.getName());
			    }
			}
			System.out.println();
		    }
		} catch(RootClassNotFound e) {
		    System.out.println("RootClassNotFound exception thrown.");
		}*/
		String[] cmdArray={"/bin/bash", "-c", "/griddata/gridsoft/local/skalinin/jhove2/jhove2-0.6.0/process_root.sh "+source.getFile().getName()};
		String line;
		Process p = Runtime.getRuntime().exec(cmdArray);
		BufferedReader input = new BufferedReader(new InputStreamReader(p.getInputStream()));
		while ((line = input.readLine() ) != null ){
			root_output.add(line);
		}
		return 0;
	}
	
	@ReportableProperty(order = 1, value = "ROOT output")
	public List<String> getRootOutput(){
		return this.root_output;
	}
	/*@ReportableProperty(order = 1, value = "Names of keys")
	public List<String> getKeys(){
		return this.keys;
	}

	@ReportableProperty(order=2, value = "Names of branches")
	public List<String> getBranches(){
		return this.branches;
	}

	@ReportableProperty(order=3, value = "Names of leaves")
        public List<String> getLeaves(){
                return this.leaves;
        }*/
}
